"use strict";

var isValidIdentifier = require('is-valid-identifier');

var _require = require('sfc-compiler/src/helpers'),
    getAndRemoveAttr = _require.getAndRemoveAttr,
    getBindingAttr = _require.getBindingAttr;

var _require2 = require('sfc-compiler/src/utils'),
    styleObjectName = _require2.styleObjectName;

var parseStyleText = function parseStyleText(cssText) {
  var res = {};
  var listDelimiter = /;(?![^(]*\))/g;
  var propertyDelimiter = /:(.+)/;
  cssText.split(listDelimiter).forEach(function (item) {
    if (item) {
      var tmp = item.split(propertyDelimiter);
      tmp.length > 1 && (res[tmp[0].trim()] = tmp[1].trim());
    }
  });
  return res;
};

function transformNode(el, options) {
  // <view style="color: red;" />
  var staticStyle = getAndRemoveAttr(el, 'style');

  if (staticStyle) {
    // { "color": "red" }
    el.staticStyle = JSON.stringify(parseStyleText(staticStyle));
  }
  /**
   * <view :style="styleObject" />
   * or
   * <view :style="{ color: 'red' }" />
   */


  var styleBinding = getBindingAttr(el, 'style', false
  /* getStatic */
  );

  if (styleBinding) {
    el.styleBinding = isValidIdentifier(styleBinding) ? 'this.' + styleBinding : styleBinding;
  }
}

function genData(el, state) {
  var data = '';

  if (!el.staticStyle && !el.styleBinding && !el.classNameStyle) {
    return data;
  } // _cx(styleBinding, staticStyle, ?styleObject, ?classNames);


  var styleBinding = el.styleBinding ? el.styleBinding : 'null';
  var staticStyle = el.staticStyle ? el.staticStyle : 'null';

  if (state.options && state.options.cssInJS) {
    var classNameStyle = el.classNameStyle ? el.classNameStyle : 'null';
    data += "style:_cx(" + styleBinding + "," + staticStyle + "," + styleObjectName + "," + classNameStyle + "),";
  } else {
    data += "style:_cx(" + styleBinding + "," + staticStyle + "),";
  }

  return data;
}

module.exports = {
  staticKeys: ['style'],
  transformNode: transformNode,
  genData: genData
};