"use strict";

var htmlparser = require('htmlparser2');

function getDomObject(html) {
  var handler = new htmlparser.DomHandler();
  var parser = new htmlparser.Parser(handler, {
    xmlMode: true
  });
  parser.parseComplete(html);
  return handler.dom;
}

function innerHTML(dom) {
  return htmlparser.DomUtils.getOuterHTML(dom, {
    xmlMode: true
  });
}

module.exports = function parse(html) {
  var dom = getDomObject(html);
  var importLinks = [];
  var styleSheetLinks = [];
  var template = '',
      script = '',
      styles = '';
  dom.forEach(function (node) {
    var name = node.name,
        children = node.children,
        attribs = node.attribs;

    switch (name) {
      case 'link':
        var rel = attribs.rel.toLowerCase();

        if (rel === 'import') {
          importLinks.push(attribs.href);
        }

        if (rel === 'stylesheet') {
          styleSheetLinks.push(attribs.href);
        }

        break;

      case 'template':
        template = innerHTML(children);
        break;

      case 'style':
        styles += innerHTML(children);
        break;

      case 'script':
        script = innerHTML(children);
        break;

      default:
        break;
    }
  });
  return {
    template: {
      type: 'template',
      content: template,
      lang: ''
    },
    script: {
      type: 'script',
      content: script,
      lang: ''
    },
    // allow multiple style tag
    // combine all css string
    styles: {
      type: 'style',
      content: styles
    },
    importLinks: importLinks,
    styleSheetLinks: styleSheetLinks
  };
};